//+------------------------------------------------------------------+
//|                                        accurateMACD_x4Alerts.mq4 |
//|                                              Copyright 2013, Mao |
//|                                          mail_to_mao@yahoo.co.jp |
//+------------------------------------------------------------------+
// This indicator is a modified version of "Good_Macd.mq4"
// 4 Alerts (crossArrow, soundAlert, panelAlert and emailAlert) were added
// to the above indicator.

// #property copyright "Copyright  2013, Mao"
// #property link      "mail_to_mao@yahoo.co.jp"

#define indiHead "accurateMACD_cross0Arrow"

#property indicator_separate_window

#property indicator_buffers 6

#property indicator_color1 Teal
#property indicator_width1 1

#property indicator_color2 IndianRed
#property indicator_width2 1
#property indicator_style2 2

#property indicator_color3 Maroon
#property indicator_width3 3

#property indicator_color4 C'0,77,0'
#property indicator_width4 3

#property indicator_color5 DeepPink
#property indicator_width5 0

#property indicator_color6 Lime
#property indicator_width6 0

/*
#property indicator_levelcolor C'30,30,30'
#property indicator_levelstyle 0
#property indicator_level1 0
*/

  //---- input parameters
  extern string Input_Period="//--------------------------------";
  extern int fastPeriod  = 12;
  extern int slowPeriod  = 26;
  extern int signalPeriod=  9;
  
  extern string alertPara="//--------------------------------";
  extern bool showArrow=true;
  extern int alertBarNum=0;
  extern double arrowPosiAdjust=0;

  //---- buffers
  double macdBuff[], signalBuff[];
  double allHisto[], upHisto[], dnHisto[];
  double upArrow[], dnArrow[];

  //---- variables
  double alpha=0;
  double alpha_1=0;
  
  string timeFrame;

  
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
  //---- name for DataWindow and indicator subwindow label
  IndicatorShortName(indiHead+" ("+fastPeriod+", "+slowPeriod+", "+signalPeriod+")");
  // IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS)+1);
  IndicatorDigits(4);
  IndicatorBuffers(7);
  
  //---- indicators
  int buffNo=0;
    SetIndexLabel(buffNo,"MACD"); SetIndexStyle(buffNo, DRAW_LINE);
    SetIndexBuffer(buffNo, macdBuff); SetIndexDrawBegin(buffNo,slowPeriod);
  
  buffNo=1;
    SetIndexLabel(buffNo,"Signal"); SetIndexStyle(buffNo, DRAW_LINE, STYLE_DOT);
    SetIndexBuffer(buffNo, signalBuff); SetIndexDrawBegin(buffNo, slowPeriod+signalPeriod);
  
  buffNo=2;
    SetIndexLabel(buffNo,"upHisto"); SetIndexStyle(buffNo, DRAW_HISTOGRAM);
    SetIndexBuffer(buffNo, upHisto); SetIndexDrawBegin(buffNo, slowPeriod+signalPeriod);
  
  buffNo=3;
    SetIndexLabel(buffNo,"dnHisto"); SetIndexStyle(buffNo, DRAW_HISTOGRAM);
    SetIndexBuffer(buffNo, dnHisto); SetIndexDrawBegin(buffNo, slowPeriod+signalPeriod);

  buffNo=4;
    SetIndexLabel (buffNo, NULL); SetIndexStyle(buffNo, DRAW_ARROW); SetIndexArrow(buffNo, 233);
    SetIndexBuffer(buffNo, upArrow); SetIndexDrawBegin(buffNo, slowPeriod+signalPeriod);
  
  buffNo=5;
    SetIndexLabel (buffNo, NULL); SetIndexStyle(buffNo, DRAW_ARROW); SetIndexArrow(buffNo, 234);
    SetIndexBuffer(buffNo, dnArrow); SetIndexDrawBegin(buffNo, slowPeriod+signalPeriod);

  buffNo=6;
    SetIndexLabel(buffNo, NULL); SetIndexStyle(buffNo, DRAW_NONE);
    SetIndexBuffer(buffNo, allHisto);

  //----
	alpha=2.0/(signalPeriod+1.0);
	alpha_1=1.0-alpha;
  //----
  return(0);
  }
  
  
//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
  //---- 
  //----
  return(0);
  }
  

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
  int limit;
  int counted_bars = IndicatorCounted();
  //---- check for possible errors
  if(counted_bars<0) return(-1);
  //---- last counted bar will be recounted
  if(counted_bars>0) counted_bars--;
  limit=Bars-counted_bars;
  
  for(int i=limit; i>=0; i--)
    {
    macdBuff[i]=iMA(NULL, 0, fastPeriod, 0, MODE_EMA, PRICE_CLOSE, i)
               -iMA(NULL, 0, slowPeriod, 0, MODE_EMA, PRICE_CLOSE, i);
    signalBuff[i]=alpha*macdBuff[i]+alpha_1*signalBuff[i+1];
    allHisto[i]=macdBuff[i]-signalBuff[i];
    
    upHisto[i]=EMPTY_VALUE; dnHisto[i]=EMPTY_VALUE; 
    if(allHisto[i+1]<allHisto[i]) upHisto[i]=allHisto[i];
    else if(allHisto[i+1]>allHisto[i]) dnHisto[i]=allHisto[i];

    if(showArrow)
      {
      int j=i+alertBarNum;
      upArrow[j]=EMPTY_VALUE; dnArrow[j]=EMPTY_VALUE;
      if(signalBuff[j+1]>macdBuff[j+1] && signalBuff[j]<macdBuff[j])
        {
        upArrow[j]=-arrowPosiAdjust;
        }
      else if(signalBuff[j+1]<macdBuff[j+1] && signalBuff[j]>macdBuff[j])
        {
        dnArrow[j]=arrowPosiAdjust;
        }
      }
  
    }   // for_Loop End
  
  //----
  return(0);
  }

